@@ -1,5 +1,5 @@
 #' @title Drug to Treatment Mapping
-#' @description Obtains a data frame that indicates the treatment(s)
+#' @description Obtains a data frame that indicates the treatments
 #' associated with each drug.
 #'
 #' @param treatment_by_drug The indicator matrix of treatment by drug
@@ -18,9 +18,7 @@
 #'
 #' drug_name = drug_description_df$drug_name
 #' dose_unit = drug_description_df$dose_unit
-#' treatment_by_drug_df <- f_treatment_by_drug_df(
-#'   treatment_by_drug, drug_name, dose_unit)
-#' treatment_by_drug_df
+#' f_treatment_by_drug_df(treatment_by_drug, drug_name, dose_unit)
 #'
 #' @export
 f_treatment_by_drug_df <- function(
@@ -30,9 +28,9 @@ f_treatment_by_drug_df <- function(
   l = ncol(treatment_by_drug)
 
   dplyr::tibble(treatment = rep(1:k, l),
-                drug = rep(1:l, each=k),
-                drug_name = rep(drug_name[1:l], each=k),
-                dose_unit = rep(dose_unit[1:l], each=k),
+                drug = rep(1:l, each = k),
+                drug_name = rep(drug_name[1:l], each = k),
+                dose_unit = rep(dose_unit[1:l], each = k),
                 included = as.logical(treatment_by_drug)) %>%
     dplyr::filter(.data$included) %>%
     dplyr::select(.data$treatment, .data$drug, .data$drug_name,
@@ -40,8 +38,8 @@ f_treatment_by_drug_df <- function(
 }
 
 
-#' @title Drug Demand Prediction
-#' @description Obtains drug demand prediction via modeling and
+#' @title Drug Demand Forecasting
+#' @description Obtains drug demand forecasting via modeling and
 #' simulation.
 #'
 #' @param df A data frame for subject-level enrollment and event data,
@@ -66,64 +64,118 @@ f_treatment_by_drug_df <- function(
 #'   It must be specified at the design stage. It will be replaced with
 #'   the observed information at the analysis stage.
 #' @param treatment_by_drug The indicator matrix of treatment by drug
-#'   combinations.
+#'   combinations. It must be specified at the design stage. It will
+#'   be replaced with the observed information at the analysis stage.
 #' @param dosing_schedule_df A data frame providing dosing schedule
 #'   information. It contains the following variables: \code{drug},
-#'   \code{target_days}, \code{target_kits}, and \code{max_cycles}.
+#'   \code{target_days}, \code{target_dose}, and \code{max_cycles}.
 #' @param model_k0 The model for the number of skipped
 #'   visits between randomization and the first drug dispensing visit.
+#'   Options include "constant", "poisson", "zip" for zero-inflated
+#'   Poisson, and "nb" for negative binomial.
 #' @param model_t0 The model for the gap time between randomization
 #'   and the first drug dispensing visit when there is no visit skipping.
+#'   Options include "constant", "exponential", "weibull",
+#'   "log-logistic", and "log-normal".
 #' @param model_ki The model for the number of skipped
 #'   visits between two consecutive drug dispensing visits.
+#'   Options include "constant", "poisson", "zip" for zero-inflated
+#'   Poisson, and "nb" for negative binomial.
 #' @param model_di The model for the dispensed doses at drug
-#'   dispensing visits.
+#'   dispensing visits. Options include "constant",
+#'   "lm" for linear model, and "lme" for linear mixed-effects model.
 #' @param pilevel The prediction interval level.
 #' @param nyears The number of years after the data cut for prediction.
-#' @param nreps The number of replications for simulation.
-#' @param n.cores.max The maximum number of cores to use for parallel
-#'   computing. The actual number of cores used will be the minimum of
-#'   \code{n.cores.max} and half of the detected number of cores.
+#' @param ncores_max The maximum number of cores to use for parallel
+#'   computing. The actual number of cores used is the minimum of
+#'   \code{ncores_max} and half of the detected number of cores.
 #' @param showplot A Boolean variable that controls whether or not to
 #'   show the drug dispensing model fit and drug demand prediction
 #'   plots. It defaults to \code{TRUE}.
 #'
-#' @return A list with the following components:
+#' @return For design-stage drug demand forecasting, a list with the
+#' following components:
 #'
-#' * \code{common_time_model} A Boolean variable that indicates whether
-#' a common time model is used for drug dispensing visits.
+#' * \code{dosing_pred_pp}: A data frame for dosing summary by drug and
+#'   time point per protocol. It includes the following variables: \code{drug},
+#'   \code{drug_name}, \code{dose_unit}, \code{t}, \code{n}, \code{pilevel},
+#'   \code{lower}, \code{upper}, \code{mean}, and \code{var}.
 #'
-#' * \code{fit_k0} The model fit for the number of skipped
-#' visits between randomization and the first drug dispensing visit.
+#' * \code{dosing_pred_plot}: A plot object for dosing prediction.
 #'
-#' * \code{fit_t0} The model fit for the gap time between randomization
-#' and the first drug dispensing visit when there is no visit skipping.
+#' For analysis-stage drug demand forecasting, a list with the
+#' following components:
 #'
-#' * \code{fit_t1} The model fit for the gap time between randomization
-#' and the first drug dispensing visit when there is visit skipping.
+#' * \code{trialsdt}: The trial start date.
 #'
-#' * \code{fit_ki} The model fit for the number of skipped
-#' visits between two consecutive drug dispensing visits.
+#' * \code{cutoffdt}: The cutoff date.
 #'
-#' * \code{fit_ti} The model fit for the gap time between two
-#' consecutive drug dispensing visits.
+#' * \code{dosing_summary_t0}: A data frame for the cumulative doses
+#'   dispensed before the cutoff date. It contains the following
+#'   variables: \code{drug}, \code{drug_name}, \code{dose_unit}, and
+#'   \code{cum_dose_t0}.
 #'
-#' * \code{fit_di} The model fit for the dispensed doses at drug
-#' dispensing visits.
+#' * \code{cum_dispense_plot}: The step plot for the cumulative doses
+#'   dispensed for each drug.
 #'
-#' * \code{dosing_subject} A data frame for the observed and imputed
-#' subject-level dosing records.
+#' * \code{bar_t0_plot}: The bar chart for the time between
+#'   randomization and the first drug dispensing visit.
 #'
-#' * \code{dosing_pred_df} A data frame for dosing summary by drug and
-#' time point.
+#' * \code{bar_ti_plot}: The bar chart for the gap time between two
+#'   consecutive drug dispensing visits.
 #'
-#' * \code{dosing_pred_pp} A data frame for dosing summary by drug and
-#' time point per protocol.
+#' * \code{bar_di_plot}: The bar chart for the doses dispensed at drug
+#'   dispensing visits.
 #'
-#' * \code{dosing_pred_plot} A plot object for dosing prediction.
+#' * \code{common_time_model}: A Boolean variable that indicates
+#'   whether a common time model is used for drug dispensing visits.
+#'
+#' * \code{fit_k0}: The model fit for the number of skipped
+#'   visits between randomization and the first drug dispensing visit.
+#'
+#' * \code{fit_t0}: The model fit for the gap time between
+#'   randomization and the first drug dispensing visit when there is
+#'   no visit skipping.
+#'
+#' * \code{fit_t1}: The model fit for the gap time between
+#'   randomization and the first drug dispensing visit when there is
+#'   visit skipping.
+#'
+#' * \code{fit_ki}: The model fit for the number of skipped
+#'   visits between two consecutive drug dispensing visits.
+#'
+#' * \code{fit_ti}: The model fit for the gap time between two
+#'   consecutive drug dispensing visits.
+#'
+#' * \code{fit_di}: The model fit for the dispensed doses at drug
+#'   dispensing visits.
+#'
+#' * \code{dosing_subject}: A data frame for the observed and imputed
+#'   subject-level dosing records for the first iteration. It includes
+#'   the following variables: \code{drug}, \code{drug_name}, \code{dose_unit},
+#'   \code{usubjid}, \code{treatment}, \code{treatment_description},
+#'   \code{arrivalTime}, \code{time}, \code{day}, \code{dose},
+#'   \code{cum_dose}, \code{row_id}, \code{subject_type}, \code{imputed},
+#'   \code{trialsdt}, \code{cutoffdt}, \code{randdt}, \code{adt}, and
+#'   \code{date}.
+#'
+#' * \code{dosing_pred_df}: A data frame for dosing summary by drug and
+#'   time point. It includes the following variables: \code{drug},
+#'   \code{drug_name}, \code{dose_unit}, \code{t}, \code{n}, \code{pilevel},
+#'   \code{lower}, \code{upper}, \code{mean}, \code{var}, and \code{date}.
+#'
+#' * \code{dosing_pred_pp}: A data frame for dosing summary by drug and
+#'   time point per protocol. It includes the following variables: \code{drug},
+#'   \code{drug_name}, \code{dose_unit}, \code{t}, \code{n}, \code{pilevel},
+#'   \code{lower}, \code{upper}, \code{mean}, \code{var}, and \code{date}.
+#'
+#' * \code{dosing_pred_plot}: A plot object for dosing prediction.
 #'
 #' @author Kaifeng Lu, \email{kaifenglu@@gmail.com}
 #'
+#' @seealso \code{\link{f_fit_t0}}, \code{\link{f_fit_ki}},
+#' \code{\link{f_fit_ti}}, \code{\link{f_fit_di}}
+#'
 #' @examples
 #'
 #' \donttest{
@@ -148,11 +200,10 @@ f_treatment_by_drug_df <- function(
 #'
 #' tictoc::tic("drug demand prediction")
 #'
-#' a <- f_drug_demand(
+#' drug_demand <- f_drug_demand(
 #'   df = df2,
 #'   newEvents = pred$event_pred$newEvents,
 #'   visitview = visitview2,
-#'   treatment_by_drug = treatment_by_drug,
 #'   dosing_schedule_df = dosing_schedule_df,
 #'   model_k0 = "zip",
 #'   model_t0 = "log-logistic",
@@ -160,13 +211,12 @@ f_treatment_by_drug_df <- function(
 #'   model_di = "lme",
 #'   pilevel = 0.95,
 #'   nyears = 1,
-#'   nreps = 200,
-#'   n.cores.max = 2,
+#'   ncores_max = 2,
 #'   showplot = FALSE)
 #'
 #' tictoc::toc()
 #'
-#' a$dosing_pred_plot
+#' drug_demand$dosing_pred_plot
 #' }
 #'
 #' @export
@@ -181,76 +231,41 @@ f_drug_demand <- function(
     model_t0 = "exponential",
     model_ki = "zip",
     model_di = "lme",
-    pilevel = 0.9,
-    nyears = 4,
-    nreps = 500,
-    n.cores.max = 10,
+    pilevel = 0.95,
+    nyears = 2,
+    ncores_max = 10,
     showplot = TRUE) {
 
   if (is.null(newEvents)) {
     stop("newEvents must be provided.")
   }
 
+  if (is.null(dosing_schedule_df)) {
+    stop("dosing_schedule_df must be provided.")
+  }
+
+  if (!is.null(visitview) && is.null(df)) {
+    stop("df must be provided if visitview is provided.")
+  }
+
+  nreps = length(unique(newEvents$draw))
+
   # trial start date and cutoff date
   if (!is.null(df)) {
     trialsdt = df$trialsdt[1]
     cutoffdt = df$cutoffdt[1]
-    df <- df %>%
-      dplyr::mutate(arrivalTime = as.numeric(.data$randdt - trialsdt + 1))
   }
 
-
-  # set up drug/subject/date drug dispensing data
+  # set up drug/subject/day drug dispensing data
   if (!is.null(visitview)) {
-    vf <- visitview %>%
-      dplyr::inner_join(df, by = "usubjid") %>%
-      dplyr::mutate(day = as.numeric(.data$date - .data$randdt + 1)) %>%
-      dplyr::select(.data$drug, .data$drug_name, .data$dose_unit,
-                    .data$usubjid, .data$treatment,
-                    .data$treatment_description, .data$arrivalTime,
-                    .data$time, .data$event, .data$dropout,
-                    .data$day, .data$dispensed_quantity) %>%
-      dplyr::group_by(.data$drug, .data$drug_name, .data$dose_unit,
-                      .data$usubjid, .data$treatment,
-                      .data$treatment_description, .data$arrivalTime,
-                      .data$time, .data$event, .data$dropout,
-                      .data$day) %>%
-      dplyr::summarise(dose = sum(.data$dispensed_quantity),
-                       .groups = "drop_last") %>%
-      dplyr::mutate(cum_dose = cumsum(.data$dose)) %>%
-      dplyr::group_by(.data$drug, .data$drug_name, .data$dose_unit,
-                      .data$usubjid) %>%
-      dplyr::mutate(row_id = dplyr::row_number())
+    observed <- f_dose_observed(df, visitview, showplot = showplot)
+    vf = observed$vf
+    treatment_by_drug_df = observed$treatment_by_drug_df
 
+    dosing_summary_t = observed$dosing_summary_t %>%
+      dplyr::mutate(pilevel = pilevel)
 
-    # obtain the observed time points relative to trial start
-    t_df <- vf %>% dplyr::mutate(t1 = .data$arrivalTime + .data$day - 1)
-    t_obs <- sort(unique(t_df$t1))
-
-    # obtain the subset of dosing records before each observed time point
-    dosing_subject_t <- dplyr::tibble(t = t_obs) %>%
-      dplyr::cross_join(vf) %>%
-      dplyr::filter(.data$arrivalTime + .data$day - 1 <= .data$t) %>%
-      dplyr::group_by(.data$drug, .data$drug_name, .data$dose_unit,
-                      .data$t, .data$usubjid) %>%
-      dplyr::mutate(cum_dose = cumsum(.data$dose)) %>%
-      dplyr::slice(dplyr::n())
-
-    # tally the doses across patients
-    dosing_summary_t <- dosing_subject_t %>%
-      dplyr::group_by(.data$drug, .data$drug_name, .data$dose_unit,
-                      .data$t) %>%
-      dplyr::summarise(n = sum(.data$cum_dose), .groups = "drop_last") %>%
-      dplyr::mutate(pilevel = pilevel, lower = NA, upper = NA,
-                    mean = .data$n, var = 0)
-
-    # obtain the cumulative doses up to cutoff
-    dosing_summary_t0 <- dosing_summary_t %>%
-      dplyr::group_by(.data$drug, .data$drug_name, .data$dose_unit) %>%
-      dplyr::slice(dplyr::n()) %>%
-      dplyr::rename(cum_dose_t0 = .data$n) %>%
-      dplyr::select(.data$drug, .data$drug_name, .data$dose_unit,
-                    .data$cum_dose_t0)
+    dosing_summary_t0 = observed$dosing_summary_t0
 
     # extract drug description from observed data
     drug_description_df <- dosing_summary_t0 %>%
@@ -265,8 +280,10 @@ f_drug_demand <- function(
   drug_name = drug_description_df$drug_name
   dose_unit = drug_description_df$dose_unit
 
-  treatment_by_drug_df <- f_treatment_by_drug_df(
-    treatment_by_drug, drug_name, dose_unit)
+  if (is.null(visitview)) {
+    treatment_by_drug_df = f_treatment_by_drug_df(
+      treatment_by_drug, drug_name, dose_unit)
+  }
 
   # time points after cutoff to impute dosing data
   t0 = ifelse(is.null(df), 1, as.numeric(cutoffdt - trialsdt + 1))
@@ -280,6 +297,23 @@ f_drug_demand <- function(
 
   # dosing prediction based on modeling and simulation
   if (!is.null(visitview)) {
+    # add date information for prediction per protocol
+    dosing_pred_pp <- dosing_pred_pp %>%
+      dplyr::mutate(date = as.Date(.data$t - 1, origin = trialsdt))
+
+    # model fit to the observed drug dispensing data
+    fit <- f_dispensing_models(vf, dosing_schedule_df,
+                               model_k0, model_t0, model_ki, model_di,
+                               nreps, showplot)
+
+    # impute drug dispensing data for ongoing and new patients
+    a <- f_dose_draw(df, vf, newEvents, treatment_by_drug_df,
+                     fit$common_time_model,
+                     fit$fit_k0, fit$fit_t0, fit$fit_t1,
+                     fit$fit_ki, fit$fit_ti, fit$fit_di,
+                     t0, t, ncores_max)
+
+
     # dosing summary for subjects who discontinued treatment before cutoff
     dosing_subject_stopped <- vf %>% dplyr::filter(.data$event == 1)
 
@@ -291,19 +325,6 @@ f_drug_demand <- function(
       dplyr::summarise(total_dose_a = sum(.data$cum_dose),
                        .groups = "drop_last")
 
-    # model fit to the observed drug dispensing data
-    fit <- f_dispensing_models(dosing_schedule_df$target_days, vf,
-                               model_k0, model_t0, model_ki, model_di,
-                               nreps, showplot)
-
-    # impute drug dispensing data for ongoing and new patients
-    a <- f_dosing_draw(df, vf, newEvents, treatment_by_drug_df,
-                       fit$common_time_model,
-                       fit$fit_k0, fit$fit_t0, fit$fit_t1,
-                       fit$fit_ki, fit$fit_ti, fit$fit_di, t0, t,
-                       n.cores.max)
-
-
     # subject level dosing data for the first simulation run
     dosing_subject <- dosing_subject_stopped %>%
       dplyr::select(-c(.data$event, .data$dropout)) %>%
@@ -322,16 +343,19 @@ f_drug_demand <- function(
       dplyr::arrange(.data$drug, .data$drug_name, .data$dose_unit,
                      .data$usubjid, .data$day) %>%
       dplyr::mutate(
-        trialsdt = trialsdt, cutoffdt = cutoffdt,
+        trialsdt = trialsdt,
+        cutoffdt = cutoffdt,
         randdt = as.Date(.data$arrivalTime - 1, origin = trialsdt),
         adt = as.Date(.data$time - 1, origin = .data$randdt),
         date = as.Date(.data$day - 1, origin = .data$randdt))
 
     # dosing summary by drug, t, draw
     dosing_summary <- a$dosing_summary_new %>%
-      dplyr::inner_join(dosing_summary_stopped,
+      dplyr::right_join(dosing_summary_stopped,
                         by = c("drug", "drug_name", "dose_unit")) %>%
-      dplyr::mutate(total_dose = .data$total_dose_a + .data$total_dose_b)
+      dplyr::mutate(total_dose = .data$total_dose_a +
+                      ifelse(is.na(.data$total_dose_b), 0,
+                             .data$total_dose_b))
 
     # dosing overview by drug, t
     dosing_overview <- dosing_summary %>%
@@ -362,9 +386,6 @@ f_drug_demand <- function(
                      .data$t) %>%
       dplyr::mutate(date = as.Date(.data$t - 1, origin = trialsdt))
 
-    # add date information for prediction per protocol
-    dosing_pred_pp <- dosing_pred_pp %>%
-      dplyr::mutate(date = as.Date(.data$t - 1, origin = trialsdt))
   }
 
 
@@ -377,24 +398,22 @@ f_drug_demand <- function(
 
       fig[[j]] <- plotly::plot_ly() %>%
         plotly::add_lines(
-          data = dfb_pp, x = ~t, y = ~n, name = "median prediction pp",
-          line = list(width=2)) %>%
+          data = dfb_pp, x = ~t, y = ~n, name = "median prediction protocol",
+          line = list(width = 2)) %>%
         plotly::add_ribbons(
           data = dfb_pp, x = ~t, ymin = ~lower, ymax = ~upper,
-          fill = "tonexty", line = list(width=0),
-          name = "prediction interval pp") %>%
+          fill = "tonexty", line = list(width = 0),
+          name = "prediction interval protocol") %>%
         plotly::layout(
           xaxis = list(title = "Days since trial start", zeroline = FALSE),
           yaxis = list(title = paste0("Doses to dispense ",
                                       "(", dfb_pp$dose_unit[1], ")"),
                        zeroline = FALSE),
-          legend = list(x = 0, y = 1.05, yanchor = "bottom",
-                        orientation = 'h'),
           annotations = list(
             x = 0.5, y = 1,
             text = paste0("<b>", dfb_pp$drug_name[1], "</b>"),
             xanchor = "center", yanchor = "bottom",
-            showarrow = FALSE, xref='paper', yref='paper'))
+            showarrow = FALSE, xref = 'paper', yref = 'paper'))
     }
   } else {
     for (j in 1:l) {
@@ -407,38 +426,36 @@ f_drug_demand <- function(
 
       fig[[j]] <- plotly::plot_ly() %>%
         plotly::add_lines(
-          data = dfa, x = ~date, y = ~n, name = "observed",
-          line = list(shape="hv", width=2)) %>%
+          data = dfa, x = ~date, y = ~n, line = list(shape ="hv", width = 2),
+          name = "observed") %>%
         plotly::add_lines(
-          data = dfb, x = ~date, y = ~n, name = "median prediction",
-          line = list(width=2)) %>%
+          data = dfb, x = ~date, y = ~n, line = list(width = 2),
+          name = "median prediction model") %>%
         plotly::add_ribbons(
           data = dfb, x = ~date, ymin = ~lower, ymax = ~upper,
-          fill = "tonexty", line = list(width=0),
-          name = "prediction interval") %>%
+          fill = "tonexty", line = list(width = 0),
+          name = "prediction interval model") %>%
         plotly::add_lines(
-          data = dfb_pp, x = ~date, y = ~n, name = "median prediction pp",
-          line = list(width=2)) %>%
+          data = dfb_pp, x = ~date, y = ~n, line = list(width = 2),
+          name = "median prediction protocol") %>%
         plotly::add_ribbons(
           data = dfb_pp, x = ~date, ymin = ~lower, ymax = ~upper,
-          fill = "tonexty", line = list(width=0),
-          name = "prediction interval pp") %>%
+          fill = "tonexty", line = list(width = 0),
+          name = "prediction interval protocol") %>%
         plotly::add_lines(
           x = rep(cutoffdt, 2), y = c(min(dfa$n), max(dfb_pp$upper)),
-          name = "cutoff", line = list(dash="dash"),
-          showlegend = FALSE) %>%
+          line = list(dash = "dash"), showlegend = FALSE,
+          name = "cutoff") %>%
         plotly::layout(
           xaxis = list(title = "", zeroline = FALSE),
           yaxis = list(title = paste0("Doses to dispense ",
                                       "(", dfb_pp$dose_unit[1], ")"),
                        zeroline = FALSE),
-          legend = list(x = 0, y = 1.05, yanchor = "bottom",
-                        orientation = 'h'),
           annotations = list(
             x = 0.5, y = 1,
             text = paste0("<b>", dfb_pp$drug_name[1], "</b>"),
             xanchor = "center", yanchor = "bottom",
-            showarrow = FALSE, xref='paper', yref='paper'))
+            showarrow = FALSE, xref = 'paper', yref = 'paper'))
 
       if (j==1) {
         fig[[j]] <- fig[[j]] %>%
@@ -451,7 +468,6 @@ f_drug_demand <- function(
     }
   }
 
-
   if (showplot) print(fig)
 
   # output results
@@ -459,7 +475,13 @@ f_drug_demand <- function(
     list(dosing_pred_pp = dosing_pred_pp,
          dosing_pred_plot = fig)
   } else {
-    list(common_time_model = fit$common_time_model,
+    list(trialsdt = trialsdt, cutoffdt = cutoffdt,
+         dosing_summary_t0 = observed$dosing_summary_t0,
+         cum_dispense_plot = observed$cum_dispense_plot,
+         bar_t0_plot = observed$bar_t0_plot,
+         bar_ti_plot = observed$bar_ti_plot,
+         bar_di_plot = observed$bar_di_plot,
+         common_time_model = fit$common_time_model,
          fit_k0 = fit$fit_k0, fit_t0 = fit$fit_t0,
          fit_t1 = fit$fit_t1, fit_ki = fit$fit_ki,
          fit_ti = fit$fit_ti, fit_di = fit$fit_di,
